package com.prowidesoftware.brilliance.mx2mt.mt103;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtCreator;
import com.prowidesoftware.brilliance.mx2mt.Mx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.mx2mt.mt103.impl.*;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.brilliance.util.TemplateUtil;
import com.prowidesoftware.brilliance.util.XmlUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class Mx2Mt103Creator extends AbstractMx2MtCreator {
    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt103Creator.class);

    @Override
    public void preProcess() {
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        map.put(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("GrpHdr"));
        map.put(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, parentElementMaps.get("CdtTrfTxInf"));
        map.put(Mx2MtConstants.MT_TEMPLATE_NAME, "swift_mt_103.vm");
    }

    @Override
    public void generateHead3() {
       try {
           String head3 = "";
           Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
           String uetr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PmtId.UETR");
           if (StringUtil.isNotEmpty(uetr)) {
               head3 += "{121:"+uetr+"}";
           }
           map.put("head3", head3);
       } catch (Exception e) {
           throw new SwiftException(e.getMessage());
       }
    }

    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {
        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new Field13CGenerate());
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field23BGenerate());
        fieldsGenerateList.add(new Field23EGenerate());
        fieldsGenerateList.add(new Field26TGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field33BGenerate());
        fieldsGenerateList.add(new Field36Generate());
        fieldsGenerateList.add(new Field50AGenerate());
        fieldsGenerateList.add(new Field51AGenerate());
        fieldsGenerateList.add(new Field52AGenerate());
        fieldsGenerateList.add(new Field53AGenerate());
        fieldsGenerateList.add(new Field54AGenerate());
        fieldsGenerateList.add(new Field55AGenerate());
        fieldsGenerateList.add(new Field56AGenerate());
        fieldsGenerateList.add(new Field57AGenerate());
        fieldsGenerateList.add(new Field59Generate());
        fieldsGenerateList.add(new Field70Generate());
        fieldsGenerateList.add(new Field71AGenerate());
        fieldsGenerateList.add(new Field72Generate());
        fieldsGenerateList.add(new Field77BGenerate());
        return fieldsGenerateList;
    }

    @Override
    public void postProcess() {
        //TODO validate
    }

    @Override
    protected String getMtType() {
        return Mx2MtConstants.MT_TYPE_103;
    }
}
