package com.prowidesoftware.brilliance.mx2mt.mt103.impl;


import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.util.ArrayList;
import java.util.List;

public class Field13CGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field13CGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            List<String> filed13cList = new ArrayList<>();
            String dbtDtTmStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.SttlmTmIndctn.DbtDtTm");
            if (StringUtil.isNotEmpty(dbtDtTmStr)) {
                XMLGregorianCalendar dbtDtTm = DatatypeFactory.newInstance().newXMLGregorianCalendar(dbtDtTmStr);
                String hourTime = DateUtil.format(dbtDtTm, "HHmm");
                filed13cList.add("/SNDTIME/"+hourTime+"+0800");
            }
            String cdtDtTmStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.SttlmTmIndctn.CdtDtTm");
            if (StringUtil.isNotEmpty(cdtDtTmStr)) {
                XMLGregorianCalendar cdtDtTm = DatatypeFactory.newInstance().newXMLGregorianCalendar(cdtDtTmStr);
                String hourTime = DateUtil.format(cdtDtTm, "HHmm");
                filed13cList.add("/RNCTIME/"+hourTime+"+0800");
            }
            String clsDtTmStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.SttlmTmReq.CLSTm");
            if (StringUtil.isNotEmpty(clsDtTmStr)) {
                XMLGregorianCalendar clsDtTm = DatatypeFactory.newInstance().newXMLGregorianCalendar(clsDtTmStr);
                String hourTime = DateUtil.format(clsDtTm, "HHmm");
                filed13cList.add("/CLSTIME/"+hourTime+"+0800");
            }
            map.put("field13CList", filed13cList);
        } catch(Exception e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
