package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1，若节点<BICFI>有值，则设置52 option为“A”，同时BIC取此栏位；
 * 2，若节点<IBAN>有值，则作为52的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为52的账号；否则直接取<Othr><Id>，则作为52的账号。
 * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *  * 1，若节点<Nm>有值，则设置52 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则和节点与52A相同
 *  * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field52AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field52AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field52Option = "";
            String field52 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgt.FinInstnId.BICFI");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgt.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field52Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field52 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field52 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)){
                field52Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field52 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field52 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field52 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field52Option", field52Option);
            map.put("field52", field52);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}