package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * //1，若节点<BICFI>有值，则设置55 option为“A”，同时BIC取此栏位
 *         //2，若节点<IBAN>有值，则作为55的账号；
 *         //3，若节点<Othr><Id>有值，则作为55的账号；
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><BICFI>
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgtAcct><Id><IBAN>
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgtAcct><Id><Othr><Id>
*          * //1，若节点<Nm>有值，则设置55 option为“D”，且名称取自此节点，而地址需要合并截取？？？
*  *       //2，账号取值规则和节点与55A相同
*  *      //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><Nm>
*  *      //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field55AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field55AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field55Option = "";
            String field55 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgt.FinInstnId.BICFI");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgt.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field55Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field55 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field55 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)){
                field55Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field55 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field55 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field55 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field55Option", field55Option);
            map.put("field55", field55);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }

    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgtAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.ThrdRmbrsmntAgtAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}
