package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * //1，若节点<BICFI>有值，则设置57 option为“A”，同时BIC取此栏位；
 *         //2，若节点<IBAN>有值，则作为57A的账号；
 *         //3，若节点<Othr><Id>有值，则作为57A的账号；
 *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><BICFI>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><IBAN>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><Id>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *              * //如果<CdtrAgt><FinInstnId><BICFI>为空，且<CdtrAgt><FinInstnId><Nm>为空，
 *  *         // 但节点<IBAN>有值，或者节点<Othr><Id>有值，则设置57 option为“C”
 *  *         //若节点<Id><Othr><Id>不为空，则取“//CH”+此节点作为账号，否则取节点<MmbId>作为账号：
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><ClrSysId><Cd>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><ClrSysId><Prtry>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><MmbId>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><Id>
 *       *         //1，若节点<Nm>有值，则设置57 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  *         //2，账号取值规则和节点与57A相同
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><Nm>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class Field57AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field57AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field57Option = "";
            String field57 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgt.FinInstnId.BICFI");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgt.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field57Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field57 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field57 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)){
                field57Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field57 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field57 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field57 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            } else if (StringUtil.isNotEmpty(account)){
                field57Option = "C";
                String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgtAcct.Id.IBAN");
                String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgtAcct.Id.Othr.Id");
                String mmbId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgt.FinInstnId.ClrSysMmbId.MmbId");
                //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
                if (StringUtil.isNotEmpty(otherId)) {
                    account = "//CH" + otherId;
                } else if (StringUtil.isNotEmpty(mmbId)) {
                    account = "/" + mmbId;
                } else {
                    account = "/" + iban;
                }
                if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                    field57 = account.substring(0, 35);
                }
            }
            map.put("field57Option", field57Option);
            map.put("field57", field57);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgtAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAgtAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}
